#include "gdipAPI.h"
#include <math.h>
#include <stdio.h>

/*************************
  GDIP interface
**************************/

GdipAPI* new_GdipAPI(){
  return new GdipAPI();
}

void set_configurations(GdipAPI* api, double start[3], double end[3], double radius){
  State q1(start[0], start[1], start[2]);
  State q2(end[0], end[1], end[2]);
  api->maneuver = Dubins(q1, q2, radius);
}

void set_configurations_dip(GdipAPI* api, double point1[2], double interval1[2], double point2[2], double interval2[2], double radius){
  AngleInterval int1(Point(point1[0], point1[1]), interval1[0], interval1[1]);
  AngleInterval int2(Point(point2[0], point2[1]), interval2[0], interval2[1]);
  api->maneuver = Dubins(int1, int2, radius);
}

void set_configurations_gdip(GdipAPI* api, double point1[2], double interval1[2], double radius1, double point2[2], double interval2[2], double radius2, double radius){
  AngleInterval int1(Point(point1[0], point1[1]), interval1[0], interval1[1]);
  AngleInterval int2(Point(point2[0], point2[1]), interval2[0], interval2[1]);
  api->maneuver = Dubins(int1, int2, radius, radius1, radius2);
}


double get_length(GdipAPI* api){
  return api->maneuver.length;
}

void sample_state_to_tmp(GdipAPI* api, double distance){
  api->tmp_state = api->maneuver.getState(distance);
}

double get_tmp_x(GdipAPI* api){
  return api->tmp_state.point.x;
}

double get_tmp_y(GdipAPI* api){
  return api->tmp_state.point.y;
}

double get_tmp_theta(GdipAPI* api){
  return api->tmp_state.ang;
}

void destruct(GdipAPI* api){
  delete api;
}
